<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\NewCode;
use Teknisa\SM\Service\Sm00004000ContratoLocalAtendService;
use Teknisa\SM\Service\Sm00004000ItemContratoService;

class Sm00004000ContratoLocalAtendController extends SMCrud
{
    protected $dataSourceName = 'fct_relacestruturacont_widget';
    protected $newCode;
    protected $serviceLocAtendCont;
    protected $itensContratoService;

    public function __construct ($crudService, $environment, NewCode $newCode, Sm00004000ContratoLocalAtendService $serviceLocAtendCont, Sm00004000ItemContratoService $itensContratoService) {
        parent::__construct($crudService, $environment);
        $this->newCode              = $newCode;
        $this->serviceLocAtendCont  = $serviceLocAtendCont;
        $this->itensContratoService = $itensContratoService;
    }

    public function save(Request $request, Response $response) {
        try{
            $rows = $request->getDataSet()->getRows();
            if ($rows[0]["__is_new"]) {
                foreach($rows as $chave => &$row) {
                    $nrEstruturam  = $row["NRESTRUTURAM"];
                    $nrSeqContrato = $row["NRSEQCONTRATO"];
                    
                    $valida = $this->serviceLocAtendCont->validaLocalAtend($nrEstruturam, $nrSeqContrato);
                    if ($valida == false) {
                        $row["NRRELACESTRUTCONT"] = $this->newCode->getCodeByTableName("NRRELACESTRUTCONT", $this->environment->getNrOrg(), 1, 10, true);
                        $row["IDRELACESTRUTURA"] = "CONTRATO";
                        if ($row["IDCONVERTIDOCLIENTE"] == "N") {
                            $row["IDLOCALTOMADORNF"] = "N";
                        }
                    }
                    else {
                        unset($rows[$chave]);
                    }
                }
                
                unset($row);
        
                $novoRequest = new Request\DataSet(new DataSet($this->dataSourceName, $rows), $request->getRoutePath(), $request->getMethod(), $request->getUserId());
                $response    = new Response();
                
                parent::save($novoRequest, $response);
                $erro = $response->getError();
                if ($erro) {
                    throw new \Exception($erro->getMessage(), $erro->getErrorCode());
                }
            }
            else {
                parent::save($request, $response);
                $erro = $response->getError();
                if ($erro) {
                    throw new \Exception($erro->getMessage(), $erro->getErrorCode());
                }
            }
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
        
    }
   
     /**
     * A função "deletarPaieFilhos" exclui  o local de Atendimento 
     * e os filhos.
     */
    
    public function deletarPaieFilhos(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $nrEstruturam = $row["NRESTRUTURAM"];
            $nrSeqContrato = $row["NRSEQCONTRATO"];
            
            $this->serviceLocAtendCont->deletarPaieFilhos($nrEstruturam, $nrSeqContrato);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }  


    
    /*=========================================================================================================================
     | À PARTIR DAQUI ESTÃO AS FUNÇÕES REFERENTES À INSERÇÃO DOS ITENS DE FORMAÇÃO DE PREÇO, QUE UTILIZAM O MESMO DATA SOURCE |
     =========================================================================================================================*/
    
    /**
     *  A função "insereItemFormPrec" organiza os dados para inserção da 
     *  formação de preço nos itens de contrato (feita no Service);
     */
    
    public function insereItemFormPrec(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
           //insere todos os itens um por um 
            $servicos        = $row["SERVICOS"];
            $soma            = $row["SOMA"];
            $contrato        = $row["CONTRATO"];
            $nrtipoorcamento = $row["NRTIPOORCAMENTO"];
            $locaisAtend     = $row["NRESTRUTURAM"];
            //setar nulo
            if(!is_array($locaisAtend)){
                $locaisAtend= array(null);
            }
            foreach($locaisAtend as $nrestruturam){
                $nrorg = $this->environment->getNrOrg();
                $cdoperador = $this->environment->getCdOperador();
                
                $itensInseridos = $this->serviceLocAtendCont->insereItemFormPrec($servicos, $soma, $contrato, $nrtipoorcamento, $nrestruturam);
                foreach ($itensInseridos as $item) {
                    $this->itensContratoService->inserirItemNegociacaoValoresContrato($item, $nrorg, $cdoperador);
                    
                    if (isset($item["INSERIR_MEDIDA"]) && $item["INSERIR_MEDIDA"] == true) {
                        $this->itensContratoService->inserirOuAtualizarMedidaVariavelServico($item, $nrorg, $cdoperador);
                    }
                }
                
                $response->addDataSet(new DataSet($this->dataSourceName, $itensInseridos));
            }
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * valida os dados de rateio do item
    **/
    public function validarDadosRateioContrato(Request\Row $request, Response $response) {
        try {
            $row     = $request->getRow();
            $nrorg   = $this->environment->getNrOrg();
            $rateios = null;
            
            $nrseqcontrato   = $row["NRSEQCONTRATO"];
            $nrtipoorcamento = $row["NRTIPOORCAMENTO"];
            
            $geraRateio = $this->serviceLocAtendCont->verificaGeraRateioContrato($nrtipoorcamento, $nrorg);
            if($geraRateio && array_key_exists("NRESTRUTURAM", $row)) {
                $nrestruturam = $row["NRESTRUTURAM"];
                $rateios = $this->serviceLocAtendCont->validarDadosRateioContrato($nrseqcontrato, $nrorg, $nrestruturam, $nrtipoorcamento);
            }
            
            $response->addDataSet(new DataSet($this->dataSourceName, array($rateios)));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}
?>